#define _GNU_SOURCE

#include <sys/types.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <unistd.h>

#include "sandbox.h"

/*
	init_ChildDataStruct
*/
void init_ChildDataStruct(struct childDataStruct *data){
	data->cwDir 		= getcwd(NULL, 0);
	data->shmData 		= NULL;
	data->writeDir 		= NULL;
	data->forwardSocket = NULL;

	data->forwardTCP.enabled	= false;
	data->forwardTCP.socket 	= (-1);

	data->userGroup		= NULL;
	data->numUserGroups = 0;
}

/*
	clear_ChildDataStruct_WriteDir
*/
void clear_ChildDataStruct_WriteDir(struct childDataStruct *data){
	struct writeDirEntry *writeDir;
	while ((writeDir = data->writeDir) != NULL){
		data->writeDir = writeDir->next;
		
		if (writeDir->uniqueID)	free(writeDir->uniqueID);
		if (writeDir->path) 	free(writeDir->path);
		free(writeDir);
	}
}

/*
	clear_ChildDataStruct_ForwardSocket
*/
void clear_ChildDataStruct_ForwardSocket(struct childDataStruct *data){
	struct forwardSocketEntry *forwardSocket;

	while ((forwardSocket = data->forwardSocket) != NULL){
		data->forwardSocket = forwardSocket->next;

		/* the sockets are not created yet at the beginning */
		if (forwardSocket->socket != (-1))
			close(forwardSocket->socket);

		if (forwardSocket->uniqueID)	free(forwardSocket->uniqueID);
		if (forwardSocket->path) 		free(forwardSocket->path);
		free(forwardSocket);
	}

	if (data->forwardTCP.socket != (-1)){
		close(data->forwardTCP.socket);
		data->forwardTCP.socket = (-1);
	}
}

/*
	clear_ChildDataStruct
*/
void clear_ChildDataStruct(struct childDataStruct *data){
	if (data->cwDir)	free(data->cwDir);
	if (data->shmData) 	munmap(data->shmData, sizeof(struct shmDataStruct));
	if (data->userGroup) free(data->userGroup);
	data->cwDir 		= NULL;
	data->shmData 		= NULL;
	data->userGroup 	= NULL;
	clear_ChildDataStruct_WriteDir(data);
	clear_ChildDataStruct_ForwardSocket(data);
}

/*
	free_StringList
*/
void free_StringList(struct stringListEntry **list){
	struct stringListEntry *entry;
	while ((entry = *list) != NULL){
		*list = entry->next;

		if (entry->str) free(entry->str);
		free(entry);
	}
}

/*
	alloc_StringListEntry
*/
struct stringListEntry* alloc_StringListEntry(const char *str){
	struct stringListEntry *entry;
	char *copyStr;

	/* get the full path */
	copyStr = str_malloc(str);
	if (!copyStr)
		return NULL; /* already prints an error message */

	entry = malloc(sizeof(struct stringListEntry));
	if (!entry){
		perror("Failed to malloc() stringListEntry");
		free(copyStr);
		return NULL;
	}

	/* return the datastructure */
	entry->next 		= NULL;
	entry->str 			= copyStr;
	return entry;
}